# Ladybug: A Plugin for Environmental Analysis (GPL)
# This file is part of Ladybug.
#
# Copyright (c) 2020, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Ladybug; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Perform simple arithmetic operations between Data Collections. For example,
adding two Data Collections together, subtracting one collection from another,
or multiplying/dividing a data in a collection by a factor.
-
Note that Data Collections must be aligned in order for this component to run
successfully.
-
Using this component will often be much faster and more elegant compared to
deconstructing the data collection, performing the operation with native
Grasshopper components, and rebuilding the collection.
-

    Args:
        _data_1: The first Data Collection in the operation. If the operator is
            not commutative, this collection comes before the operator. For
            example, in subtraction, this is the collection being subtracted from.
            This can also be a list of Data Collections that align with _data_2.
            It cal also be a single number that will be added, multiplied, etc.
            to all of _data_2.
        _data_2: The second Data Collection in the operation. If the operator is
            not commutative, this collection comes after the operator. For
            example, in subtraction, this is the collection being subtracted with.
            This can also be a list of Data Collections that align with _data_1.
            It cal also be a single number that will be added, multiplied, etc.
            to all of _data_1.
        _operator_: Text for the operator to use between the two Data Collections.
            Valid examples include (+, -, *, /). By default this is + for addition.
        type_: Optional text for a new "type" key in the Data Collection's metadata.
            This will usually show up in most Ladybug visualiztions and it should
            usually change for most types of operations.
    
    Returns:
        data: A Ladybug data collection object derived from the operation between
            the two data inputs.
"""

ghenv.Component.Name = "LB Arithmetic Operation"
ghenv.Component.NickName = 'ArithOp'
ghenv.Component.Message = '0.1.0'
ghenv.Component.Category = 'Ladybug'
ghenv.Component.SubCategory = '1 :: Analyze Weather Data'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

try:
    from itertools import izip as zip  # python 2
except ImportError:
    pass  # some future time when GHPython upgrades to python 3


if all_required_inputs(ghenv.Component):
    # build the arithmetic statement
    operator = '+' if _operator_ is None else _operator_
    statement = 'data_1 {} data_2'.format(operator)

    # perform the arithmetic operation
    data = []
    for data_1, data_2 in zip(_data_1, _data_2):
        result = eval(statement, {'data_1': data_1, 'data_2': data_2})  # I love Python!

        # try to replace the data collection type
        try:
            if type_:
                result.header.metadata['type'] = type_
            elif 'type' in result.header.metadata:
                del result.header.metadata['type']
        except AttributeError:
            pass  # result was not a data collection; just return it anyway
        data.append(result)


# Ladybug: A Plugin for Environmental Analysis (GPL)
# This file is part of Ladybug.
#
# Copyright (c) 2020, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Ladybug; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Convert a DataCollection to IP values.
-

    Args:
        _data_si: A DataCollection in SI units.
    
    Returns:
        data_ip: The DataCollection in IP units.
"""

ghenv.Component.Name = 'LB To IP'
ghenv.Component.NickName = 'ToIP'
ghenv.Component.Message = '0.1.0'
ghenv.Component.Category = 'Ladybug'
ghenv.Component.SubCategory = '4 :: Extra'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from ladybug.datacollection import BaseCollection
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    assert isinstance(_data_si, BaseCollection), \
        '_data_si must be a Data Collection. Got {}.'.format(type(_data_si))
    data_ip = _data_si.to_ip()


# Ladybug: A Plugin for Environmental Analysis (GPL)
# This file is part of Ladybug.
#
# Copyright (c) 2020, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Ladybug; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>


"""
Create a set of parameters that define the acceptable conditions of the
Adaptive thermal comfort model.
-
These parameters can be plugged into any of the components that compute
Adaptive thermal comfort.
-

    Args:
        _ashrae_or_en15251_: A boolean to note whether to use the ASHRAE-55 neutral
            temperature function (True) or the EN-15251 function (False).
            Note that this input will also determine default values for many of
            the other properties of this object.
        _neutral_offset_:  The number of degrees Celcius from the neutral temperature
            where the input operative temperature is considered acceptable.
            The default is 2.5C when the neutral temperature function is ASHRAE-55
            and 3C when the neutral temperature function is EN-15251.
            You may want to use the set_neutral_offset_from_ppd() or the
            set_neutral_offset_from_comfort_class() methods on this object to set
            this value using ppd from the ASHRAE-55 standard or comfort classes
            from the EN-15251 standard respectively.
        _avgm_or_runmean_: A boolean to note whether the prevailing outdoor
            temperature is computed from the average monthly temperature (True) or
            a weighted running mean of the last week (False).  The default is True
            when the neutral temperature function is ASHRAE-55 and False when the
            neutral temperature function is EN-15251.
        _discr_or_cont_vel_: A boolean to note whether discrete
            categories should be used to assess the effect of elevated air speed
            (True) or whether a continuous function should be used (False).
            The default is True when the neutral temperature function is ASHRAE-55
            and False when the neutral temperature function is EN-15251.
        _cold_prevail_limit_: A number indicating the prevailing outdoor
            temperature below which acceptable indoor operative temperatures
            flatline. The default is 10C when the neutral temperature function is
            ASHRAE-55 and 15C when the neutral temperature function is EN-15251.
            This number cannot be greater than 22C and cannot be less than 10C.
        _conditioning_: A number between 0 and 1 that represents how "conditioned"
            vs. "free-running" the building is.
                0 = free-running (completely passive with no air conditioning)
                1 = conditioned (no operable windows and fully air conditioned)
            The default is 0 since both the ASHRAE-55 and EN-15251 standards prohibit
            the use of adaptive comfort methods when a heating/cooling system is active.
            When set to a non-zero number, a neutral temperature function for
            heated/cooled operation derived from the SCATs database will be used.
            For more information on how adaptive comfort methods can be applied to
            conditioned buildings, see the neutral_temperature_conditioned function
            in the ladybug_comfort documentation.
    
    Returns:
        adapt_par: An Adaptive comfort parameter object that can be plugged into
            any of the components that compute Adaptive thermal comfort.
"""

ghenv.Component.Name = 'LB Adaptive Comfort Parameters'
ghenv.Component.NickName = 'AdaptPar'
ghenv.Component.Message = '0.1.0'
ghenv.Component.Category = 'Ladybug'
ghenv.Component.SubCategory = '4 :: Extra'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:
    from ladybug_comfort.parameter.adaptive import AdaptiveParameter
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_comfort:\n\t{}'.format(e))


adapt_par = AdaptiveParameter(_ashrae_or_en15251_, _neutral_offset_,
                              _avgm_or_runmean_, _discr_or_cont_vel_,
                              _cold_prevail_limit_, _conditioning_)



# Ladybug: A Plugin for Environmental Analysis (GPL)
# This file is part of Ladybug.
#
# Copyright (c) 2020, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Ladybug; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>


"""
Create a custom legend for any set of data or range. Creating a legend with this
component allows for a bit more flexibility than what can be achieved by working
with the legends automatically output from different studies.
-

    Args:
        _values: A list of numerical data that the legend refers to (or just the
            minimum and maximum numerical values of this data).  If the original
            numerical data is hooked up, the legend's maximum and minimum values
            will be set by the max and min of the data set.
        _base_plane_: An optional plane or point to set the location of the
            legend. (Default: Rhino origin - (0, 0, 0))
        title_: A text string representing a legend title. Legends are usually
            titled with the units of the data.
        legend_par_: Optional legend parameters from the  Legend Parameters component.
    
    Returns:
        mesh: A colored mesh for the legend colors.
        title_obj: A text object for the  legend title.
        label_objs: An array of text objects for the label text.
        label_text: An array of text strings for the label text.
        legend_par: The input legend parameters with defaults filled for
            unset properties.
"""

ghenv.Component.Name = "LB Create Legend"
ghenv.Component.NickName = 'CreateLegend'
ghenv.Component.Message = '0.1.0'
ghenv.Component.Category = 'Ladybug'
ghenv.Component.SubCategory = '4 :: Extra'
ghenv.Component.AdditionalHelpFromDocStrings = '1'


try:
    from ladybug.legend import Legend, LegendParameters
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from ladybug_rhino.togeometry import to_plane
    from ladybug_rhino.fromobjects import legend_objects
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set default values
    legend_par_ = legend_par_.duplicate() if legend_par_ is not None else \
        LegendParameters()
    if _base_plane_:
        legend_par_.base_plane = to_plane(_base_plane_)
    legend_par_.title = title_

    # create the legend
    legend = Legend(_values, legend_par_)
    legend_par = legend.legend_parameters

    # separate all of the outputs from this component
    rhino_objs = legend_objects(legend)
    mesh = rhino_objs[0]
    title_obj = rhino_objs[1]
    label_objs = rhino_objs[2:]
    label_text = legend.segment_text



# Ladybug: A Plugin for Environmental Analysis (GPL)
# This file is part of Ladybug.
#
# Copyright (c) 2020, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Ladybug; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Calculate Mean Radiant Temperature (MRT) as a result of outdoor shortwave
solar shining directly onto people as well as longwave radiant exchange
with the sky.
-
This component uses the SolarCal model of ASHRAE-55 to estimate the effects
of shortwave solar and a simple sky exposure method to determine longwave
radiant exchange.
-

    Args:
        _location: A Ladybug Location object.
        _dir_norm_rad: Hourly Data Collection with the direct normal solar
            irradiance in W/m2.
        _diff_horiz_rad: Hourly Data Collection with diffuse horizontal solar
            irradiance in W/m2.
        _longwave_mrt: A single number or an hourly data collection with the
            long wave mean radiant temperature around the person in degrees C.
            This includes the temperature of the ground and any other surfaces
            between the person and their view to the sky.
            Typically, dry bulb temperature is used when such surface
            temperatures are unknown.
        fract_body_exp_: A single number between 0 and 1 or a data collection
            representing the fraction of the body exposed to direct sunlight.
            Note that this does not include the body’s self-shading; only the
            shading from surroundings.
            Default is 1 for a person standing in an open area.
        sky_exposure_: A single number between 0 and 1 or a data collection
            representing the fraction of the sky vault in occupant’s view.
            Default is 1 for a person standing in an open area.
        _ground_ref_: A single number between 0 and 1 or a data collection
            that represents the reflectance of the floor. Default is for 0.25
            which is characteristic of concrete.
        _window_trans_: A Data Collection or number between 0 and 1 that
                represents the broadband solar transmittance of the window through which
                the sun is coming. Such values tend to be slightly less than the
                SHGC. Values might be as low as 0.2 and could be as high as 0.85
                for a single pane of glass. Default is 0.4 assuming a double pane
                window with a relatively mild low-e coating.
        _solar_body_par_: Optional Solar Body Parameter object to account for
            properties of the human geometry.
        _run: Set to True to run the component.
    
    Returns:
        report: Reports, errors, warnings, etc.
        erf: Data collection of effective radiant field (ERF) in W/m2.
        dmrt: Data collection of mean radiant temperature delta in C.
        mrt: Data collection of mean radiant temperature in C.
"""

ghenv.Component.Name = 'LB Indoor Solar MRT'
ghenv.Component.NickName = 'IndoorSolarMRT'
ghenv.Component.Message = '0.1.0'
ghenv.Component.Category = 'Ladybug'
ghenv.Component.SubCategory = '1 :: Analyze Weather Data'
ghenv.Component.AdditionalHelpFromDocStrings = '5'

try:
    from ladybug_comfort.collection.solarcal import IndoorSolarCal
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run is True:
    solar_mrt_obj = IndoorSolarCal(_location, _dir_norm_rad,
                                   _diff_horiz_rad, _longwave_mrt,
                                   fract_body_exp_, sky_exposure_,
                                   _ground_ref_, _window_trans_, _solar_body_par_)

    erf = solar_mrt_obj.effective_radiant_field
    dmrt = solar_mrt_obj.mrt_delta
    mrt = solar_mrt_obj.mean_radiant_temperature


# Ladybug: A Plugin for Environmental Analysis (GPL)
# This file is part of Ladybug.
#
# Copyright (c) 2020, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Ladybug; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Orient a series of geometries to the active viewport camera.
-

    Args:
        _geo: A series of geometries to be oriented to the camera of the active
            Rhino viewport.
        _position_: A point to be used as the origin around which the the geometry
            will be oriented. If None, the lower left corner of the bounding box
            around the geometry will be used.
        refresh_: Connect a Grasshopper "button" component to refresh the orientation
            upon hitting the button. You can also connect a Grasshopper "Timer"
            component to update the view in real time as you navigate through
            the Rhino scene.
    
    Returns:
        geo: The input geometry that has been oriented to the camera of the active
            Rhino viewport.
"""

ghenv.Component.Name = 'LB Orient to Camera'
ghenv.Component.NickName = 'OrientCam'
ghenv.Component.Message = '0.1.1'
ghenv.Component.Category = 'Ladybug'
ghenv.Component.SubCategory = '4 :: Extra'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import Rhino.Geometry as rg
import Rhino.Display as rd

try:
    from ladybug_rhino.grasshopper import all_required_inputs
    from ladybug_rhino.text import TextGoo
    from ladybug_rhino.viewport import camera_oriented_plane
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


def bounding_box_origin(geometry):
    """Get the origin of a bounding box around a list of geometry.

    Args:
        geometry: A list of geometry for which the bounding box origin will
            be computed.
    """
    b_box = geometry[0].GetBoundingBox(False)
    for geo in geometry[1:]:
        if isinstance(geo, rd.Text3d):
            b_box = rg.BoundingBox.Union(b_box, geo.BoundingBox)
        else:
            b_box = rg.BoundingBox.Union(b_box, geo.GetBoundingBox(False))
    return b_box.Corner(True, True, True)


if all_required_inputs(ghenv.Component):
    # set the default position if it is None
    origin = bounding_box_origin(_geo)
    pt = origin if _position_ is None else _position_

    # get a plane oriented to the camera
    oriented_plane = camera_oriented_plane(pt)

    # orient the input geometry to the plane facing the camera
    base_plane = rg.Plane(origin, rg.Vector3d(0, 0, 1))
    xform = rg.Transform.PlaneToPlane(base_plane, oriented_plane)
    geo = []
    for rh_geo in _geo:
        if isinstance(rh_geo, rd.Text3d):
            geo.append(TextGoo(rh_geo).Transform(xform))
        else:
            new_geo = rh_geo.Duplicate()
            new_geo.Transform(xform)
            geo.append(new_geo)
